Package: cellxgenedp
Title: Discover and Access Single Cell Data Sets in the CELLxGENE Data
        Portal
Version: 1.15.0
Authors@R: 
    c(person(
        given = "Martin",
        family = "Morgan",
        role = c("aut", "cre"),
        email = "mtmorgan.bioc@gmail.com",
        comment = c(ORCID = "0000-0002-5874-8148")
    ),
    person("Kayla", "Interdonato", role = "aut"))
Description: The cellxgene data portal
    (https://cellxgene.cziscience.com/) provides a graphical user
    interface to collections of single-cell sequence data processed in
    standard ways to 'count matrix' summaries. The cellxgenedp package
    provides an alternative, R-based inteface, allowind data
    discovery, viewing, and downloading.
License: Artistic-2.0
Encoding: UTF-8
Collate: db.R collections.R datasets.R files.R facets.R keys.R
        cellxgene.R utilities.R publisher_metadata.R cxg.R
Depends: R (>= 4.1.0), dplyr
Imports: httr, curl, utils, tools, cli, shiny, DT, rjsoncons
Suggests: zellkonverter, SingleCellExperiment, HDF5Array, tidyr,
        BiocStyle, knitr, rmarkdown, testthat (>= 3.0.0), mockery
biocViews: SingleCell, DataImport, ThirdPartyClient
URL: https://mtmorgan.github.io/cellxgenedp/,
        https://github.com/mtmorgan/cellxgenedp
BugReports: https://github.com/mtmorgan/cellxgenedp/issues
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.2
VignetteBuilder: knitr
Config/testthat/edition: 3
git_url: https://git.bioconductor.org/packages/cellxgenedp
git_branch: devel
git_last_commit: 844e4cf
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-30
NeedsCompilation: no
Packaged: 2025-10-30 20:28:51 UTC; biocbuild
Author: Martin Morgan [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-5874-8148>),
  Kayla Interdonato [aut]
Maintainer: Martin Morgan <mtmorgan.bioc@gmail.com>
